from __future__ import unicode_literals

import logging
import os

from mopidy import config, ext

import pygst
pygst.require('0.10')

__version__ = '0.1.3'

logger = logging.getLogger(__name__)

class Extension(ext.Extension):

    dist_name = 'Mopidy-GPIO'
    ext_name = 'gpio'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        schema['debug_gpio_simulate'] = config.Boolean()
        schema['pin_button_main'] = config.Integer()
        schema['pin_button_next'] = config.Integer()
        schema['pin_button_previous'] = config.Integer()
        schema['pin_button_vol_up'] = config.Integer()
        schema['pin_button_vol_down'] = config.Integer()
        return schema

    def setup(self, registry):

        from .frontend import GPIOFrontend
        registry.add('frontend', GPIOFrontend)

        from .backend import GPIOBackend
        registry.add('backend', GPIOBackend)



import time
import logging

import RPi.GPIO as GPIO

longpress_time = 0.3

logger = logging.getLogger(__name__)


class GPIOManager():

    def __init__(self, frontend, pins):

        self.frontend = frontend

        # Variables to control if it is a longpress
		self.cur_down = 0
        self.down_time_previous = 0
        self.down_time_next = 0
        self.down_time_main = 0
        self.down_time_vol_up = 0
        self.down_time_vol_down = 0

        # GPIO Mode
        GPIO.setmode(GPIO.BCM)

        # Next Button
        #GPIO.setup(pins['pin_button_next'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        #GPIO.add_event_detect(pins['pin_button_next'], GPIO.BOTH, callback=self.next, bouncetime=30)

        # Previous Button
        #GPIO.setup(pins['pin_button_previous'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        #GPIO.add_event_detect(pins['pin_button_previous'], GPIO.BOTH, callback=self.previous, bouncetime=30)

        # Volume Up Button
        GPIO.setup(pins['pin_button_vol_up'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['pin_button_vol_up'], GPIO.BOTH, callback=self.vol_up, bouncetime=30)

        # Volume Down Button
        GPIO.setup(pins['pin_button_vol_down'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['pin_button_vol_down'], GPIO.BOTH, callback=self.vol_down, bouncetime=30)

        # Main Button
        #GPIO.setup(pins['pin_button_main'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        #GPIO.add_event_detect(pins['pin_button_main'], GPIO.BOTH, callback=self.main, bouncetime=30)

    def previous(self, channel):
        if GPIO.input(channel) == 1:
            if self.down_time_previous + longpress_time > time.time():
                self.frontend.input({'key': 'previous', 'long': False})
            else:
                self.frontend.input({'key': 'previous', 'long': True})
        else:
            self.down_time_previous = time.time()

    def next(self, channel):
        if GPIO.input(channel) == 1:
            if self.down_time_next + longpress_time > time.time():
                self.frontend.input({'key': 'next', 'long': False})
            else:
                self.frontend.input({'key': 'next', 'long': True})
        else:
            self.down_time_next = time.time()

    def main(self, channel):
        if GPIO.input(channel) == 1:
            if self.down_time_main + longpress_time > time.time():
                self.frontend.input({'key': 'main', 'long': False})
            else:
                self.frontend.input({'key': 'main', 'long': True})
        else:
            self.down_time_main = time.time()

    def vol_up(self, channel):
		logger.info('GPIO: vol_up Triggered')

        if GPIO.input(channel) == 1:
            if self.down_time_vol_up + longpress_time > time.time():
                self.frontend.input({'key': 'volume_up', 'long': False})
            else:
                self.frontend.input({'key': 'volume_up', 'long': True})
        else:
            self.down_time_vol_up = time.time()

    def vol_down(self, channel):
		logger.info('GPIO: vol_down Triggered')

        if GPIO.input(channel) == 1:
            if self.down_time_vol_down + longpress_time > time.time():
                self.frontend.input({'key': 'volume_down', 'long': False})
            else:
                self.frontend.input({'key': 'volume_down', 'long': True})
        else:
            self.down_time_vol_down = time.time()



import os
import time
import logging
import traceback
import socket
import RPi.GPIO as GPIO
import pykka

from mopidy import core

logger = logging.getLogger(__name__)

class GPIOFrontend(pykka.ThreadingActor, core.CoreListener):

    def __init__(self, config, core):
        super(GPIOFrontend, self).__init__()
        self.core = core
		self.backend = pykka.ActorRegistry.get_by_class_name("GPIOBackend")[0]
		self.gpio_manager = GPIOManager(self, config['gpio'])
		logger.info('GPIO: Frontend started')

	def input(self, input_event):
	    logger.info('GPIO: Input Triggered')
        try:
            if input_event['key'] == 'volume_up':
                if input_event['long']:
                    self.repeat()
                else:
                    current = self.core.playback.volume.get()
                    current += 10
                    self.backend.tell({'action': 'set_volume', 'value': current})
            elif input_event['key'] == 'volume_down':
                if input_event['long']:
                    current = 0
                else:
                    current = self.core.playback.volume.get()
                    current -= 10
                self.backend.tell({'action': 'set_volume', 'value': current})
            elif input_event['key'] == 'main' and input_event['long'] and self.menu:
                self.exit_menu()
            else:
                if self.menu:
                    self.main_menu.input(input_event)
                else:
                    self.manage_input(input_event)

        except Exception:
            traceback.print_exc()
			
    def repeat(self):
        if self.menu:
            self.main_menu.repeat()
        else:
            self.speak_current_song(self.core.playback.current_tl_track.get())